﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Verse;
using RimWorld;
using UnityEngine;
using RimWorld.Planet;



namespace Immortals
{
    public class Immortal_Component : GameComponent
    {
        public Immortal_Component(Game game)
        {
            loaded = false;
        }

        public override void GameComponentUpdate()
        {
            int tickMult = Find.TickManager.TicksGame - lastTick;
            lastTick = Find.TickManager.TicksGame;
            if (tickMult < 50 && tickMult > 0)
            {
                List<Pawn> pawnsToDelete = new List<Pawn>();
                //Work on dead immortals
                if (deadPawns.Count > 0)
                {
                    List<Pawn> pawnsToRez = new List<Pawn>();
                    foreach (Pawn pawn in deadPawns)
                    {
                        if (pawn.Corpse != null && pawn.health.hediffSet.HasHead)
                        {
                            float healSpeed = (baseHealSpeed * settings.baseHealSpeed)* tickMult;
                            CompRottable compRottable = pawn.Corpse.GetComp<CompRottable>();
                            Hediff hediff;
                            if (compRottable != null)
                            {
                                if (compRottable.RotProgress < 80000)
                                    compRottable.RotProgress -= 0.5f * tickMult;
                                else if (compRottable.RotProgress < 140000)
                                    compRottable.RotProgress -= 0.75f * tickMult;
                                else
                                    compRottable.RotProgress -= 0.9f * tickMult;
                            }
                            if (pawn.health.hediffSet.HasHediff(immortalHediff))
                            {
                                hediff = pawn.health.hediffSet.GetHediffs<Immortal_Hediff>().First();
                                if (hediff != null)
                                {
                                    healSpeed = healSpeed * hediff.Severity;
                                    if (hediff.Severity <= 0.5f)
                                    {
                                        healSpeed = healSpeed * settings.firstDeathHealFactor;
                                        healSpeed = healSpeed * 2;
                                    }
                                }
                            }
                            

                            IEnumerable<Hediff> heDiffsEnumerable = pawn.health.hediffSet.hediffs.Where(hd => NeedToRecover(hd, pawn));
                            hediff = null;
                            if (heDiffsEnumerable.Count() != 0)
                                hediff = heDiffsEnumerable.RandomElement();

                            if (hediff != null)
                            {
                                if (SlowToRecover(hediff, true))
                                    hediff.Severity -= healSpeed / 10;
                                else
                                {
                                    hediff.Severity -= healSpeed;
                                }
                            }
                            else
                            {
                                pawnsToRez.Add(pawn);
                            }
                        }
                        else
                         pawnsToDelete.Add(pawn);

                    }
                    if (pawnsToRez.Count > 0)
                    {
                        foreach (Pawn pawn in pawnsToRez)
                        {
                            List<BodyPartRecord> partsToRestore = new List<BodyPartRecord>();
                            foreach (Hediff hediff in pawn.health.hediffSet.hediffs)
                            {
                                if (hediff.GetType() == typeof(Hediff_MissingPart))
                                {
                                    Hediff_MissingPart partDiff = hediff as Hediff_MissingPart;
                                    if (partDiff.Part != null)
                                    {
                                        if (NeedPart(partDiff.Part))
                                        {
                                            partsToRestore.Add(partDiff.Part);
                                        }
                                    }
                                }
                            }
                            foreach (BodyPartRecord part in partsToRestore)
                            {
                                pawn.health.RestorePart(part);
                            }
                            Pawn_HealthTracker health = pawn.health;
                            HediffSet hediffSet = health.hediffSet;

                            pawn.health.hediffSet = new HediffSet(pawn);
                            ImmunityHandler immunity = health.immunity;
                            pawn.health.immunity = new ImmunityHandler(pawn);

                            List<BodyPartRecord> restoreParts = new List<BodyPartRecord>();

                            Thing storage = pawn.Corpse.StoringThing();
                            if (storage != null)
                            {
                                GenSpawn.Spawn(pawn.Corpse, storage.Position, Find.CurrentMap);
                            }
                            ResurrectionUtility.Resurrect(pawn);

                            foreach (Hediff hediff in hediffSet.hediffs)
                            {
                                if (hediff.TendableNow())
                                {
                                    HediffWithComps hediffWithComps = hediff as HediffWithComps;
                                    if (hediffWithComps != null)
                                    {
                                        HediffComp_TendDuration hediffComp_TendDuration = hediffWithComps.TryGetComp<HediffComp_TendDuration>();

                                        hediffComp_TendDuration.tendQuality = 0f;
                                        hediffComp_TendDuration.tendTicksLeft = Find.TickManager.TicksGame;
                                    }
                                }
                                if (hediff.GetType() == typeof(Hediff_MissingPart))
                                {
                                    Hediff_MissingPart partDiff = hediff as Hediff_MissingPart;
                                    partDiff.IsFresh = false;
                                }
                            }
                            pawn.health.hediffSet = hediffSet;
                            pawn.TakeDamage(dmg);
                            pawn.health.immunity = immunity;

                            Hediff imHediff = pawn.health.hediffSet.GetHediffs<Immortal_Hediff>().First();
                            if (imHediff.Severity <= 0.5f)
                                imHediff.Severity = 1f;
                            else
                            {
                                switch (settings.deathEffectOnVal)
                                {
                                    case -1: imHediff.Severity -= 0.1f; break;
                                    case 1: imHediff.Severity += 0.1f; break;
                                }
                            }

                            deadPawns.Remove(pawn);
                            AddImmortal(pawn, true);
                        }
                    }
                    
                }

                //Work on living immortals
                if (immortalPawns.Count() > 0)
                {
                    foreach (Pawn pawn in immortalPawns)
                    {
                        if (pawn != null)
                        {
                            IEnumerable<Hediff> slowDownEnumerable = pawn.health.hediffSet.hediffs;
                            Hediff hediff;
                            Hediff imDiff = pawn.health.hediffSet.GetFirstHediffOfDef(immortalHediff);
                            if (imDiff.Severity > 0.5f)
                            {
                                foreach (Hediff slowDiff in slowDownEnumerable)
                                {
                                    if (!settings.canGetDisease)
                                    {
                                        if (ToCure(slowDiff))
                                        {
                                            slowDiff.Heal(100f);
                                        }
                                    }
                                    if (SlowToRecover(slowDiff, true))
                                    {
                                        
                                        if (slowDiff.Severity > settings.diseaseMaxProg)
                                        {
                                            slowDiff.Severity = settings.diseaseMaxProg;
                                        }

                                    }
                                }
                            }
                            float healSpeed = (baseHealSpeed * settings.baseHealSpeed) * tickMult;
                            
                            if (pawn.health.hediffSet.HasHediff(immortalHediff))
                            {
                                hediff = pawn.health.hediffSet.GetHediffs<Immortal_Hediff>().First();
                                if (hediff != null)
                                {
                                    healSpeed = healSpeed * hediff.Severity;
                                    if (hediff.Severity > 0.5f)
                                    {
                                        IEnumerable<Hediff> heDiffsEnumerable = pawn.health.hediffSet.hediffs.Where(hd => NeedToHeal(hd, false));
                                        hediff = null;
                                        if (heDiffsEnumerable.Count() != 0)
                                            hediff = heDiffsEnumerable.RandomElement();
                                        
                                        if (hediff != null)
                                        {
                                            if (!SlowToRecover(hediff, true))
                                                hediff.Severity -= healSpeed;
                                        }
                                    }
                                }
                            }

                            
                        }
                        else pawnsToDelete.Add(pawn);
                    }
                }
                if (pawnsToDelete.Count() > 0)
                {
                    foreach (Pawn pawn in pawnsToDelete)
                    {
                        CheckQuickening(pawn);
                        deadPawns.Remove(pawn);
                    }
                }
                
                
                rareTick += tickMult;
            }

            if (rareTick > 1000)
            {
                rareTick -= 1000;

                //Rare stuff yo

                
            }

        }
        
        private bool checkDistance(IntVec3 vec1, IntVec3 vec2, int distance)
        {
            int x;
            int z;

            x = vec1.x - vec2.x;
            z = vec1.z - vec2.z;

            x = Mathf.Abs(x);
            z = Mathf.Abs(z);

            if (x <= distance && z <= distance)
                return true;
            return false;

        }


        private int GetDistance(IntVec3 vec1, IntVec3 vec2)
        {
            int x;
            int z;

            x = vec1.x - vec2.x;
            z = vec1.z - vec2.z;

            x = Mathf.Abs(x);
            z = Mathf.Abs(z);

            return x + z;

        }

        public override void StartedNewGame()
        {
            StartUp();
        }

        public override void LoadedGame()
        {
            StartUp();

        }

        private void SetRecovery(Pawn pawn)
        {
            //if (pawn.health.hediffSet.has)
            deadPawns.Add(pawn);
            float minHeal = 0.25f;
            float maxHeal = 1f;
            if (pawn.health.hediffSet.GetHediffs<Immortal_Hediff>().First().Severity < 1)
            {
                minHeal = 0f;
                maxHeal = 0.5f;
            }
            Dictionary<float, float> healValues = new Dictionary<float, float>();
            foreach (Hediff hediff in pawn.health.hediffSet.hediffs)
            {
                float val = Rand.Range(minHeal, maxHeal);
                if (hediff.def.lethalSeverity == 1)
                    val = val / 10;
                healValues.Add(hediff.loadID, val);
            }
            if (recoverValues.ContainsKey(pawn))
                recoverValues[pawn] = healValues;
            else
                recoverValues.Add(pawn, healValues);
        }

        public void AddImmortal(Pawn pawn, bool flash)
        { 
            if (pawn != null)
            {
                if (flash)
                    pawn.Map.weatherManager.eventHandler.AddEvent(new WeatherEvent_Quickening(pawn.Map, pawn.Position, 0));

                immortalPawns.Add(pawn);
                if (deadPawns.Contains(pawn))
                {
                    deadPawns.Remove(pawn);
                }
            }
        }


        public void AddDeadImmortal(Pawn pawn, bool strike = true)
        {
            if (immortalPawns.Contains(pawn))
                immortalPawns.Remove(pawn);
            if (pawn.health.hediffSet.HasHead)
            {
                if (!pawn.health.hediffSet.HasHediff(timerHediff))
                {
                    pawn.health.AddHediff(timerHediff);
                }
                pawn.health.hediffSet.GetFirstHediffOfDef(timerHediff).Severity = 3f;
                foreach (Hediff hediff in pawn.health.hediffSet.hediffs)
                {
                    if (hediff.GetType() == typeof(Hediff_MissingPart))
                    {
                        Hediff_MissingPart partDiff = hediff as Hediff_MissingPart;
                        if (partDiff.Part != null)
                        {
                            if (NeedPart(partDiff.Part))
                            {
                                pawn.health.hediffSet.GetFirstHediffOfDef(timerHediff).Severity += 14;
                            }
                        }
                    }
                }
                SetRecovery(pawn);
            }
            else
            {
                CheckQuickening(pawn);
            }
        }

        #region HealChecks

        private void CheckQuickening(Pawn pawn)
        {
            pawn.Corpse.Map.weatherManager.eventHandler.AddEvent(new WeatherEvent_Quickening(pawn.Corpse.Map, pawn.Position, 0));
            int closest = 25;
            Pawn closestPawn = null;
            foreach (Pawn other in immortalPawns)
            {
                int checkDist = GetDistance(pawn.Position, other.Position);
                if (checkDist < closest)
                {
                    closest = checkDist;
                    closestPawn = other;
                }
            }
            if (closestPawn != null)
            {
                GameConditionDef def = DefDatabase<GameConditionDef>.GetNamed("IH_Quickening");
                closestPawn.Map.weatherManager.eventHandler.AddEvent(new WeatherEvent_Quickening(pawn.Corpse.Map, pawn.Position, 0));
                closestPawn.health.hediffSet.GetFirstHediffOfDef(immortalHediff).Severity += pawn.health.hediffSet.GetFirstHediffOfDef(immortalHediff).Severity / 3;
                GameCondition quickening = GameConditionMaker.MakeCondition(def, 10000);
                closestPawn.Map.GameConditionManager.RegisterCondition(quickening);
                (quickening as GameCondition_Quickening).pawn = closestPawn;
            }
            else
            {
                GameConditionDef def = DefDatabase<GameConditionDef>.GetNamed("Flashstorm");
                pawn.Corpse.Map.weatherManager.eventHandler.AddEvent(new WeatherEvent_Quickening(pawn.Corpse.Map, pawn.Corpse.Position, 15, true));
                GameCondition flashStorm = GameConditionMaker.MakeCondition(def, 1000);
                pawn.Corpse.Map.GameConditionManager.RegisterCondition(flashStorm);
            }
        }

        private bool NeedPart(BodyPartRecord part)
        {
            
            foreach (string need in sourceNeeds)
            {
                foreach (BodyPartTagDef tag in part.def.tags)
                {
                    if (tag.vital)
                    {
                        return true;
                    }
                }
            }

            return false;
        }

        private bool NeedToHeal(Hediff hediff, bool dead)
        {
            if (hediff.def.hediffClass.Name == "Hediff_Injury")
            {
                if (!hediff.IsPermanent())
                    return true;
            }
            if (SlowToRecover(hediff, dead))
                return true;

            return false;
        }

        private bool NeedToRecover(Hediff hediff, Pawn pawn)
        {
            if (NeedToHeal(hediff, pawn.Dead))
            {
                if (!recoverValues[pawn].ContainsKey(hediff.loadID))
                {
                    return false;
                }

                if (recoverValues[pawn][hediff.loadID] <= hediff.Severity)
                {
                    return true;
                }
            }
            return false;
        }

        private bool SlowToRecover(Hediff hediff, bool dead)
        {
            if (dead)
            {
                if (hediff.def.lethalSeverity == 1)
                    return true;
                foreach (string hd in deadAilments)
                {
                    if (hediff.def.defName == hd)
                        return true;
                }
            }
            else
            {
                foreach (string hd in livingAilments)
                {
                    if (hediff.def.defName == hd)
                        return true;
                }
            }

            return false;
        }

        private bool ToCure(Hediff hediff)
        {
            if (!settings.canGetDisease)
            {
                foreach (String hd in cureAilments)
                {
                    if (hediff.def.defName == hd)
                    {
                        return true;
                    }
                }
            }

            return false;
        }

        #endregion

        private void StartUp()
        {;
            immortalPawns = new List<Pawn>();
            deadPawns = new List<Pawn>();
            
            rareTick = 0;

            settings = LoadedModManager.GetMod<ImmortalsMod>().GetSettings<ImmortalsSettings>();
            ReLoadAilments();
            DamageDef dmgDef = Verse.DefDatabase<DamageDef>.GetNamed("Blunt");
            dmg = new DamageInfo(dmgDef, 1, 0);
            sourceNeeds = new List<string>
            {
                "ConsciousnessSource",
                "BloodPumpingSource",
                "BloodFiltrationLiver"
            };

            immortalHediff = Verse.DefDatabase<HediffDef>.GetNamed("IH_Immortal");
            timerHediff = DefDatabase<HediffDef>.GetNamed("IH_DeathTimer");
            recoverValues = new Dictionary<Pawn, Dictionary<float, float>>();
            
            foreach (Pawn pawn in Find.CurrentMap.mapPawns.AllPawns)
            {
                if (pawn.health.hediffSet.HasHediff(immortalHediff))
                {
                    if (pawn.Dead)
                        AddDeadImmortal(pawn, false);
                    else
                        AddImmortal(pawn, false);
                }
            }

            foreach (Pawn pawn in Find.WorldPawns.AllPawnsAliveOrDead)
            {

                if (pawn.health.hediffSet.HasHediff(immortalHediff))
                {
                    if (pawn.Dead)
                        AddDeadImmortal(pawn, false);
                    else
                        AddImmortal(pawn, false);
                }
            }
            loaded = true;
        }

        public void ReLoadAilments()
        {
            cureAilments = new List<string>();

            if (settings.canGetDisease)
            {
                cureAilments = new List<string>()
                {
                    "WoundInfection",
                    "Flu",
                    "Plague",
                    "Malaria"
                };
            }

            deadAilments = new List<string>
            {
                "BloodLoss",
                "Hypothermia",
                "Heatstroke",
                "WoundInfection",
                "Flu",
                "Plague",
                "Malaria",
                "IH_DeathTimer"
            };

            livingAilments = new List<string>()
            {
                "BloodLoss",
                "Hypothermia",
                "Heatstroke"

            };
        }

        /* Work Priority Saving
        private void SaveWorkPriorities(Pawn pawn)
        {
            Dictionary<string, int> workValues = new Dictionary<string, int>();
            foreach (WorkTypeDef workDef in Verse.DefDatabase<WorkTypeDef>.AllDefs)
            {
                workValues.Add(workDef.defName, pawn.workSettings.GetPriority(workDef));
            }
            if (workPriorities.ContainsKey(pawn.GetUniqueLoadID()))
                workPriorities[pawn.GetUniqueLoadID()] = workValues;
            else
                workPriorities.Add(pawn.GetUniqueLoadID(), workValues);
        }

        private void LoadWorkPriorities(Pawn pawn)
        {
            string loadId = pawn.GetUniqueLoadID();
            if (workPriorities.ContainsKey(loadId))
            {
                Pawn_WorkSettings workSettings = new Pawn_WorkSettings(pawn);
                workSettings.EnableAndInitializeIfNotAlreadyInitialized();
                foreach (WorkTypeDef workDef in Verse.DefDatabase<WorkTypeDef>.AllDefs)
                {
                    if (workPriorities[loadId].ContainsKey(workDef.defName))
                    {
                        workSettings.SetPriority(workDef, workPriorities[loadId][workDef.defName]);
                    }
                    else
                        Log.Message(workDef.defName + " was not saved");
                }
                pawn.workSettings = workSettings;
            }
        }
        */

        List<Pawn> immortalPawns;
        List<Pawn> deadPawns;

        List<string> deadAilments;
        List<string> livingAilments;
        List<string> cureAilments;
        List<string> sourceNeeds;
        Dictionary<Pawn, Dictionary<float, float>> recoverValues;

        HediffDef immortalHediff;
        HediffDef timerHediff;
        DamageInfo dmg;

        int lastTick;
        int rareTick;

        ImmortalsSettings settings;

        private static float baseHealSpeed = 0.002f;

        public bool loaded = false;

    }





}
