﻿

using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using Harmony;
using RimWorld;
using UnityEngine;
using Verse;
using Verse.AI;
using Verse.AI.Group;
using Verse.Sound;
using OpCodes = System.Reflection.Emit.OpCodes;

namespace Immortals
{
    [StaticConstructorOnStartup]
    internal static class ImmortalsHarmony
    {
        static ImmortalsHarmony()
        {

            var harmony = HarmonyInstance.Create("rimworld.immortals.Fishbrains");
            harmony.PatchAll(Assembly.GetExecutingAssembly());
            HarmonyInstance.DEBUG = true;

            harmony.Patch(original: AccessTools.Method(type: typeof(PawnGenerator), name: "GenerateInitialHediffs"), prefix: null, postfix: null,
                transpiler: new HarmonyMethod(type: typeof(ImmortalsHarmony),
                name: nameof(BookIconTranspiler)));

        }
        public static IEnumerable<CodeInstruction> BookIconTranspiler(IEnumerable<CodeInstruction> instructions)
        {
            List<CodeInstruction> instructionList = instructions.ToList();
            for (int i = 0; i < instructionList.Count; i++)
            {
                CodeInstruction instruction = instructionList[i];


                if (instruction.opcode == OpCodes.Call && instruction.operand == AccessTools.Method(type: typeof(PawnAddictionHediffsGenerator), name: "GenerateAddictionsAndTolerancesFor"))
                {
                    yield return new CodeInstruction(opcode: OpCodes.Ldarg_0);
                    yield return new CodeInstruction(opcode: OpCodes.Call, operand: AccessTools.Method(type: typeof(ImmortalsHarmony), name: nameof(ImmortalsHarmony.ChanceAddImmortal)));
                }
                yield return instruction;
            }
        }



        public static void ChanceAddImmortal(Pawn pawn)
        {

            if (pawn != null)
            {
                if (pawn.health != null)
                {
                    if (pawn.health.hediffSet != null)
                    {
                        ImmortalsSettings settings = LoadedModManager.GetMod<ImmortalsMod>().GetSettings<ImmortalsSettings>();
                        float rand = Rand.RangeInclusive(0, 100);
                        Log.Message("Chance: " + rand + " / " + settings.immortalChance + "(" + settings.visibleImmortalChance + ")");
                        
                        if (rand <= settings.immortalChance || rand <= settings.visibleImmortalChance && rand != 0)
                        {
                            HediffDef immortal = DefDatabase<HediffDef>.GetNamed("IH_Immortal");
                            pawn.health.AddHediff(immortal);
                            //
                            if (rand <= settings.visibleImmortalChance)
                            {
                                //pawn.health.hediffSet.GetFirstHediffOfDef(immortal).Severity = 1f;
                            }
                        }
                    }
                    
                }
                
            }
            
        }


    }

}

