﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Verse;
using RimWorld;
using UnityEngine;
using RimWorld.Planet;

namespace Immortals
{

    public class ImmortalsSettings : ModSettings
    {
        public int deathEffectOnVal = 0;
        public int immortalChance = 5;
        public int visibleImmortalChance = 1;
        public float diseaseMaxProg = 0.99f;
        public float baseHealSpeed = 1;
        public float firstDeathHealFactor = 0.1f;
        public bool canGetDisease = true;
        public bool canDieToDisease = true;

        
        public override void ExposeData()
        {
            Scribe_Values.Look(ref deathEffectOnVal, "deathEffectOnVal", 1);
            Scribe_Values.Look(ref immortalChance, "immortalChance", 5);
            Scribe_Values.Look(ref visibleImmortalChance, "visibleImmortalChance", 1);
            Scribe_Values.Look(ref diseaseMaxProg, "diseaseMaxProg", 0.99f);
            Scribe_Values.Look(ref baseHealSpeed, "baseHealSpeed", 1f);
            Scribe_Values.Look(ref firstDeathHealFactor, "firstDeathHealFactor", 0.1f);
            Scribe_Values.Look(ref canGetDisease, "canGetDisease", true);
            Scribe_Values.Look(ref canDieToDisease, "canDieToDisease", true);



        }

    }


    public class ImmortalsMod : Mod
    {
        float snapMult = 0.25f;
        float healCurve;
        ImmortalsSettings settings;

        public ImmortalsMod(ModContentPack content) : base(content)
        {
            this.settings = GetSettings<ImmortalsSettings>();
            healCurve = settings.baseHealSpeed;
            if (healCurve > 2)
            {
                healCurve = 2 + ((healCurve - 2) / 4);
            }
        }
        public override void DoSettingsWindowContents(Rect inRect)
        {


            string defaultStr = "IH_Default".Translate();
            Listing_Standard listingStandard = new Listing_Standard();
            listingStandard.Begin(inRect);



            //listingStandard.BeginScrollView(inRect, ref scroll, ref rect);

            listingStandard.Label("IH_DeathEffectOnVal".Translate());

            if (listingStandard.RadioButton("IH_Decrease".Translate(), (settings.deathEffectOnVal == -1)))
                settings.deathEffectOnVal = -1;
            if (listingStandard.RadioButton("IH_None".Translate() + defaultStr, (settings.deathEffectOnVal == 0), 1))
                settings.deathEffectOnVal = 0;
            if (listingStandard.RadioButton("IH_Increase".Translate(), (settings.deathEffectOnVal == 1)))
                settings.deathEffectOnVal = 1;

            listingStandard.CheckboxLabeled("IH_CanGetDisease".Translate(),ref settings.canGetDisease, "IH_CanGetDiseaseTip");
            if (settings.canGetDisease)
            {
                listingStandard.Label("IH_diseaseMaxProg".Translate() + ((settings.diseaseMaxProg).ToString("0%")));
                settings.diseaseMaxProg = SnapToMult(listingStandard.Slider(settings.diseaseMaxProg, 0, 1.0f), 0.01f);
            }


            listingStandard.Label("IH_baseHealSpeed".Translate() + ((settings.baseHealSpeed).ToString("0.00x")));
            healCurve = listingStandard.Slider(healCurve, 0.05f, 4.0f);
            if (healCurve < 2)
                settings.baseHealSpeed = SnapToMult(healCurve, 0.05f);
            else
                settings.baseHealSpeed = SnapToMult(2 + ((healCurve -2) * 4),0.25f);

            listingStandard.Label("IH_firstDeathHeal".Translate() + ((settings.firstDeathHealFactor).ToString("0%")));
            settings.firstDeathHealFactor = SnapToMult(listingStandard.Slider(settings.firstDeathHealFactor, 0.01f, 2f),0.01f);


            listingStandard.Label("IH_chance".Translate() + (settings.immortalChance ));
            settings.immortalChance =(int) listingStandard.Slider(settings.immortalChance, 0, 100);

            listingStandard.Label("IH_chanceVisible".Translate() + (settings.visibleImmortalChance));
            settings.visibleImmortalChance =(int) listingStandard.Slider(settings.visibleImmortalChance, 0, 100);

            /*

            listingStandard.Label("TB_RequiredResearch".Translate("TechLevel_Archotech".Translate(), settings.archotechBaseCost) + "  " + "TB_Default".Translate("8000"));
            settings.archotechBaseCost = listingStandard.Slider(SnapToMult(settings.archotechBaseCost, 25f), 0f, 25000);

            //Insights 
            if (settings.randomInsights)
            {
                listingStandard.CheckboxLabeled("TB_RandomInsightsOn".Translate(settings.randomInsightRate * 100) + "  " + "TB_Default".Translate("60%"), ref settings.randomInsights);
                settings.randomInsightRate = listingStandard.Slider(SnapToMult(settings.randomInsightRate, 0.01f), 0f, 1);
            }
            else
                listingStandard.CheckboxLabeled("TB_RandomInsightsOff".Translate(), ref settings.randomInsights);

            //listingStandard.Label(settings.canResearchAt + "TB_CanResearchAt".Translate());
            //settings.requiredPoints = listingStandard.Slider(settings.requiredPoints, 0.1f, 1);
            */
            listingStandard.End();

            if (lastDisease)
                update = true;
            if (update)
            {
                Immortal_Component immortalComponent = Current.Game.GetComponent<Immortal_Component>();
                immortalComponent.ReLoadAilments();
            }
            

        }

        private bool PosNegNeu(int val)
        {
            return false;
        }

        public override string SettingsCategory()
        {
            return "IH_Immortals".Translate();
        }

        private float SnapToMult(float val, float mult)
        {
            val = val / mult;
            return Mathf.RoundToInt(val) * mult;
        }
        bool update = false;

        bool lastDisease = false;
    }
}
